package aexp.jsonrpclib;

import android.app.Activity;
import android.app.ListActivity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.Window;
import android.widget.ArrayAdapter;
import android.widget.Toast;
import org.alexd.jsonrpc.*;

public class JsonRpc extends ListActivity
{

// New entry
    private static final int MENU_ENTRY = 1;    
// Process list
    private static final int MENU_PROCESS = 2;    
// Clear list
    private static final int MENU_CLEAR = 3;    

    private static final int ACTIVITY_ENTRY = 1;

    static final String LOG_TAG = "JSONCALC-LIB";

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
        setContentView(R.layout.main);
        adapter = new ArrayAdapter<OpEntry>( this, R.layout.main_item );
        adapter.setNotifyOnChange( true );
        setListAdapter( adapter );
        uiHandler = new Handler();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        boolean result = super.onCreateOptionsMenu(menu);
        menu.add( Menu.NONE, MENU_ENTRY, Menu.NONE, R.string.menu_entry );
        menu.add( Menu.NONE, MENU_PROCESS, Menu.NONE, R.string.menu_process );
        menu.add( Menu.NONE, MENU_CLEAR, Menu.NONE, R.string.menu_clear );
        return result;
    }

    @Override
    public boolean onOptionsItemSelected( MenuItem item ) {
        switch ( item.getItemId() ) {
            case MENU_ENTRY: {
                    Intent i = new Intent();
                    i.setClassName( 
                        "aexp.jsonrpclib",
                        "aexp.jsonrpclib.Entry" );
                    startActivityForResult(i, ACTIVITY_ENTRY );
                    return true;
                }

            case MENU_CLEAR:
                adapter.clear();
                return true;

            case MENU_PROCESS:
                setProgressBarIndeterminateVisibility( true );
                ProcessList p = new ProcessList();
                p.start();
                return true;
        }
        return false;
    }

    @Override
    protected void onActivityResult( int requestCode,
                                        int resultCode, 
                                        Intent extras ) {
        super.onActivityResult( requestCode, resultCode, extras);
        Log.d( LOG_TAG, "onActivityResult: requestCode: "+
                    requestCode+
                    "; resultCode: "+resultCode+
                    "; extras: "+extras );
        switch(requestCode) {
            case ACTIVITY_ENTRY: {
                    if( resultCode == RESULT_OK ) {
                        double v1 = extras.getDoubleExtra( "v1",0.0 );
                        double v2 = extras.getDoubleExtra( "v2",0.0 );
                        int op = extras.getIntExtra( "op", 0 );
                        OpEntry oe = new OpEntry( v1,v2,op );
                        adapter.add( oe );
                    }
                }
                break;
        }
    }


    private String process() {
        String errMsg = null;

		for( int i = 0 ; i < adapter.getCount() ; ++i ) {
        	OpEntry entry = adapter.getItem( i );
        	JSONRPCClient client = JSONRPCClient.create( Config.APP_BASE_URI );
        	client.setConnectionTimeout(2000);
        	client.setSoTimeout(2000);
        	try  {
            	double d = client.callDouble( 
					entry.opMethod(),
					entry.getV1(),
					entry.getV2() );
				entry.setResult( d );
        	} catch (JSONRPCException e ) {
            	e.printStackTrace();
        	}
		}
        return null;
    }

    private ArrayAdapter<OpEntry> adapter;
    private Handler uiHandler;

    class ProcessList extends Thread {
        public void run() {
            String errMsg = process();
            ProcessListUIUpdate plu = new ProcessListUIUpdate( errMsg );
            uiHandler.post( plu );
        }
    }

    class ProcessListUIUpdate implements Runnable {
        ProcessListUIUpdate( String errMsg ) {
            this.errMsg = errMsg;
        }

        public void run() {
            setProgressBarIndeterminateVisibility( false );
            if( errMsg != null )
                Toast.makeText( JsonRpc.this, 
                        errMsg,
                        Toast.LENGTH_SHORT).show();
            else
                adapter.notifyDataSetChanged();
        }

        String errMsg;
    }
}

class OpEntry {
    public static final int OP_ADD = 1;
    public static final int OP_SUB = 2;
    public static final int OP_MUL = 3;
    public static final int OP_DIV = 4;


    public static final String ID_KEY = "id";
    public static final String V1_KEY = "v1";
    public static final String V2_KEY = "v2";
    public static final String JSONRPC_KEY = "jsonrpc";
    public static final String METHOD_KEY = "method";
    public static final String PARAMS_KEY = "params";
    public static final String RESULT_KEY = "result";

    OpEntry( double v1, double v2, int op ) {
            this.v1 = v1;
            this.v2 = v2;
            this.op = op;
            resultSet = false;
    }

    public String toString() {
            StringBuffer sb = new StringBuffer();
            sb.append( Double.toString( v1 ) );
            sb.append( opChar() );
            sb.append( Double.toString( v2 ) );
            if( resultSet ) {
                sb.append( '=' );
                sb.append( Double.toString( result ) );
            }
            return new String( sb );
    }

    public void setResult( double result ) {
        this.result = result;
        resultSet = true;
    }

    public char opChar() {
            char c = '?';
            switch( op ) {
                case OP_ADD:
                    c = '+';
                    break;

                case OP_SUB:
                    c = '-';
                    break;

                case OP_MUL:
                    c='*';
                    break;

                case OP_DIV:
                    c='/';
                    break;
            }
            return c;
    }

    public String opMethod() {
        String s = null;
        switch( op ) {
            case OP_ADD:
                s = "add";
                break;

            case OP_SUB:
                s = "sub";
                break;

            case OP_MUL:
                s = "mul";
                break;

            case OP_DIV:
                s ="div";
                break;
        }
        return s;
    }

	public double getV1() {
		return v1;
	}

	public double getV2() {
		return v2;
	}

    private double v1,v2,result;
    private boolean resultSet;
    private int op;
}

